#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <pthread.h>

void *Thread_sum(void *rank);

/* Global variables: accessible to all threads */
int thread_count;
int n;
double sum;
pthread_mutex_t mutex;

int main(int argc, char *argv[]) {

  long thread; /* same size as void pointer for casting */
  pthread_t *thread_handles;

  /* Check for right number of arguments */
  if (argc < 3)
    {
      printf("Usage: ./pth_pi_mutex <num threads> <n>\nAborting.\n");
      return -1;
    }
  
  /* Get number of threads from command line */
  thread_count = atoi(argv[1]);

  /* Get n for calculation of PI from command line */
  n = atoi(argv[2]);  
  
  /* create handle array */
  thread_handles = malloc (thread_count * sizeof(pthread_t));

  /* initialize mutex */
  pthread_mutex_init(&mutex, NULL);
  
  for (thread = 0; thread < thread_count; thread++)
    pthread_create(&thread_handles[thread], NULL,
		   Thread_sum, (void *)thread);
  
  for (thread = 0; thread < thread_count; thread++)
    pthread_join(thread_handles[thread], NULL);

  printf("Sum after join: %f\n", sum);
  printf("Estimation of PI: :%f\n", 4*sum);

   /* clean up */
  pthread_mutex_destroy(&mutex);
  free(thread_handles);
  return 0;
} /* main */

void *Thread_sum(void *rank) {

  long my_rank = (intptr_t) rank;
  double factor, my_sum = 0.0;
  long long i;
  long long my_n = n/thread_count;
  long long my_first_i = my_n*my_rank;
  long long my_last_i = my_first_i + my_n;

  if (my_first_i % 2 == 0)
    factor = 1.0;
  else
    factor = -1.0;

  for (i = my_first_i; i < my_last_i; i++, factor = -factor)
    my_sum += factor/(2*i+1);

  printf("[%ld] my_sum: %f\n",my_rank, my_sum);
  
  pthread_mutex_lock(&mutex);
  sum += my_sum;
  printf("[%ld] sum: %f\n",my_rank, sum);
  pthread_mutex_unlock(&mutex);
  
  return NULL;
} /* Thread_sum */
